import numpy as np
from typing import Any

AR_U: np.chararray[Any, np.dtype[np.str_]]
AR_S: np.chararray[Any, np.dtype[np.bytes_]]

reveal_type(AR_U == AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S == AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U != AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S != AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U >= AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S >= AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U <= AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S <= AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U > AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S > AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U < AR_U)  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S < AR_S)  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U * 5)  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S * [5])  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U % "test")  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S % b"test")  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.capitalize())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.capitalize())  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.center(5))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.center([2, 3, 4], b"a"))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.encode())  # E: chararray[Any, dtype[bytes_]]
reveal_type(AR_S.decode())  # E: chararray[Any, dtype[str_]]

reveal_type(AR_U.expandtabs())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.expandtabs(tabsize=4))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.join("_"))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.join([b"_", b""]))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.ljust(5))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.ljust([4, 3, 1], fillchar=[b"a", b"b", b"c"]))  # E: chararray[Any, dtype[bytes_]]
reveal_type(AR_U.rjust(5))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.rjust([4, 3, 1], fillchar=[b"a", b"b", b"c"]))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.lstrip())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.lstrip(chars=b"_"))  # E: chararray[Any, dtype[bytes_]]
reveal_type(AR_U.rstrip())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.rstrip(chars=b"_"))  # E: chararray[Any, dtype[bytes_]]
reveal_type(AR_U.strip())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.strip(chars=b"_"))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.partition("\n"))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.partition([b"a", b"b", b"c"]))  # E: chararray[Any, dtype[bytes_]]
reveal_type(AR_U.rpartition("\n"))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.rpartition([b"a", b"b", b"c"]))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.replace("_", "-"))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.replace([b"_", b""], [b"a", b"b"]))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.split("_"))  # E: ndarray[Any, dtype[object_]]
reveal_type(AR_S.split(maxsplit=[1, 2, 3]))  # E: ndarray[Any, dtype[object_]]
reveal_type(AR_U.rsplit("_"))  # E: ndarray[Any, dtype[object_]]
reveal_type(AR_S.rsplit(maxsplit=[1, 2, 3]))  # E: ndarray[Any, dtype[object_]]

reveal_type(AR_U.splitlines())  # E: ndarray[Any, dtype[object_]]
reveal_type(AR_S.splitlines(keepends=[True, True, False]))  # E: ndarray[Any, dtype[object_]]

reveal_type(AR_U.swapcase())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.swapcase())  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.title())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.title())  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.upper())  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.upper())  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.zfill(5))  # E: chararray[Any, dtype[str_]]
reveal_type(AR_S.zfill([2, 3, 4]))  # E: chararray[Any, dtype[bytes_]]

reveal_type(AR_U.count("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_S.count([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[{int_}]]

reveal_type(AR_U.endswith("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.endswith([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_U.startswith("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.startswith([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.find("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_S.find([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_U.rfind("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_S.rfind([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[{int_}]]

reveal_type(AR_U.index("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_S.index([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_U.rindex("a", start=[1, 2, 3]))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(AR_S.rindex([b"a", b"b", b"c"], end=9))  # E: ndarray[Any, dtype[{int_}]]

reveal_type(AR_U.isalpha())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isalpha())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isalnum())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isalnum())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isdecimal())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isdecimal())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isdigit())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isdigit())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.islower())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.islower())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isnumeric())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isnumeric())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isspace())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isspace())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.istitle())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.istitle())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.isupper())  # E: ndarray[Any, dtype[bool_]]
reveal_type(AR_S.isupper())  # E: ndarray[Any, dtype[bool_]]

reveal_type(AR_U.__array_finalize__(object()))  # E: None
reveal_type(AR_S.__array_finalize__(object()))  # E: None



import numpy as np

nditer_obj: np.nditer

reveal_type(np.nditer([0, 1], flags=["c_index"]))  # E: nditer
reveal_type(np.nditer([0, 1], op_flags=[["readonly", "readonly"]]))  # E: nditer
reveal_type(np.nditer([0, 1], op_dtypes=np.int_))  # E: nditer
reveal_type(np.nditer([0, 1], order="C", casting="no"))  # E: nditer

reveal_type(nditer_obj.dtypes)  # E: tuple[dtype[Any], ...]
reveal_type(nditer_obj.finished)  # E: bool
reveal_type(nditer_obj.has_delayed_bufalloc)  # E: bool
reveal_type(nditer_obj.has_index)  # E: bool
reveal_type(nditer_obj.has_multi_index)  # E: bool
reveal_type(nditer_obj.index)  # E: int
reveal_type(nditer_obj.iterationneedsapi)  # E: bool
reveal_type(nditer_obj.iterindex)  # E: int
reveal_type(nditer_obj.iterrange)  # E: tuple[builtins.int, ...]
reveal_type(nditer_obj.itersize)  # E: int
reveal_type(nditer_obj.itviews)  # E: tuple[ndarray[Any, dtype[Any]], ...]
reveal_type(nditer_obj.multi_index)  # E: tuple[builtins.int, ...]
reveal_type(nditer_obj.ndim)  # E: int
reveal_type(nditer_obj.nop)  # E: int
reveal_type(nditer_obj.operands)  # E: tuple[ndarray[Any, dtype[Any]], ...]
reveal_type(nditer_obj.shape)  # E: tuple[builtins.int, ...]
reveal_type(nditer_obj.value)  # E: tuple[ndarray[Any, dtype[Any]], ...]

reveal_type(nditer_obj.close())  # E: None
reveal_type(nditer_obj.copy())  # E: nditer
reveal_type(nditer_obj.debug_print())  # E: None
reveal_type(nditer_obj.enable_external_loop())  # E: None
reveal_type(nditer_obj.iternext())  # E: bool
reveal_type(nditer_obj.remove_axis(0))  # E: None
reveal_type(nditer_obj.remove_multi_index())  # E: None
reveal_type(nditer_obj.reset())  # E: None

reveal_type(len(nditer_obj))  # E: int
reveal_type(iter(nditer_obj))  # E: nditer
reveal_type(next(nditer_obj))  # E: tuple[ndarray[Any, dtype[Any]], ...]
reveal_type(nditer_obj.__copy__())  # E: nditer
with nditer_obj as f:
    reveal_type(f)  # E: nditer
reveal_type(nditer_obj[0])  # E: ndarray[Any, dtype[Any]]
reveal_type(nditer_obj[:])  # E: tuple[ndarray[Any, dtype[Any]], ...]
nditer_obj[0] = 0
nditer_obj[:] = [0, 1]



"""
**Note:** almost all functions in the ``numpy.lib`` namespace
are also present in the main ``numpy`` namespace.  Please use the
functions as ``np.<funcname>`` where possible.

``numpy.lib`` is mostly a space for implementing functions that don't
belong in core or in another NumPy submodule with a clear purpose
(e.g. ``random``, ``fft``, ``linalg``, ``ma``).

Most contains basic functions that are used by several submodules and are
useful to have in the main name-space.

"""
import math

from numpy.version import version as __version__

# Public submodules
# Note: recfunctions and (maybe) format are public too, but not imported
from . import mixins
from . import scimath as emath

# Private submodules
from .type_check import *
from .index_tricks import *
from .function_base import *
from .nanfunctions import *
from .shape_base import *
from .stride_tricks import *
from .twodim_base import *
from .ufunclike import *
from .histograms import *

from .polynomial import *
from .utils import *
from .arraysetops import *
from .npyio import *
from .arrayterator import Arrayterator
from .arraypad import *
from ._version import *
from numpy.core._multiarray_umath import tracemalloc_domain

__all__ = ['emath', 'math', 'tracemalloc_domain', 'Arrayterator']
__all__ += type_check.__all__
__all__ += index_tricks.__all__
__all__ += function_base.__all__
__all__ += shape_base.__all__
__all__ += stride_tricks.__all__
__all__ += twodim_base.__all__
__all__ += ufunclike.__all__
__all__ += arraypad.__all__
__all__ += polynomial.__all__
__all__ += utils.__all__
__all__ += arraysetops.__all__
__all__ += npyio.__all__
__all__ += nanfunctions.__all__
__all__ += histograms.__all__

from numpy._pytesttester import PytestTester
test = PytestTester(__name__)
del PytestTester
